# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2019


import opentracing
import wrapt

from ...log import logger
from ...singletons import agent, setup_tornado_tracer, tornado_tracer
from ...util.secrets import strip_secrets_from_query

try:
    import tornado

    # Tornado >=6.0 switched to contextvars for context management.  This requires changes to the opentracing
    # scope managers which we will tackle soon.
    # Limit Tornado version for the time being.
    if not (hasattr(tornado, 'version') and tornado.version[0] < '6'):
        logger.debug('Instana supports Tornado package versions < 6.0.  Skipping.')
        raise ImportError

    from opentracing.scope_managers.tornado import tracer_stack_context

    setup_tornado_tracer()

    def extract_custom_headers(span, headers):
        if not agent.options.extra_http_headers or not headers:
            return
        try:
            for custom_header in agent.options.extra_http_headers:
                if custom_header in headers:
                    span.set_tag("http.header.%s" % custom_header, headers[custom_header])

        except Exception:
            logger.debug("extract_custom_headers: ", exc_info=True)


    @wrapt.patch_function_wrapper('tornado.web', 'RequestHandler._execute')
    def execute_with_instana(wrapped, instance, argv, kwargs):
        try:
            with tracer_stack_context():
                ctx = None
                if hasattr(instance.request.headers, '__dict__') and '_dict' in instance.request.headers.__dict__:
                    ctx = tornado_tracer.extract(opentracing.Format.HTTP_HEADERS,
                                                 instance.request.headers.__dict__['_dict'])
                scope = tornado_tracer.start_active_span('tornado-server', child_of=ctx)

                # Query param scrubbing
                if instance.request.query is not None and len(instance.request.query) > 0:
                    cleaned_qp = strip_secrets_from_query(instance.request.query, agent.options.secrets_matcher,
                                                          agent.options.secrets_list)
                    scope.span.set_tag("http.params", cleaned_qp)

                url = "%s://%s%s" % (instance.request.protocol, instance.request.host, instance.request.path)
                scope.span.set_tag("http.url", url)
                scope.span.set_tag("http.method", instance.request.method)

                scope.span.set_tag("handler", instance.__class__.__name__)

                # Request header tracking support
                extract_custom_headers(scope.span, instance.request.headers)

                setattr(instance.request, "_instana", scope)

                # Set the context response headers now because tornado doesn't give us a better option to do so
                # later for this request.
                tornado_tracer.inject(scope.span.context, opentracing.Format.HTTP_HEADERS, instance._headers)
                instance.set_header(name='Server-Timing', value="intid;desc=%s" % scope.span.context.trace_id)

                return wrapped(*argv, **kwargs)
        except Exception:
            logger.debug("tornado execute", exc_info=True)


    @wrapt.patch_function_wrapper('tornado.web', 'RequestHandler.set_default_headers')
    def set_default_headers_with_instana(wrapped, instance, argv, kwargs):
        if not hasattr(instance.request, '_instana'):
            return wrapped(*argv, **kwargs)

        scope = instance.request._instana
        tornado_tracer.inject(scope.span.context, opentracing.Format.HTTP_HEADERS, instance._headers)
        instance.set_header(name='Server-Timing', value="intid;desc=%s" % scope.span.context.trace_id)


    @wrapt.patch_function_wrapper('tornado.web', 'RequestHandler.on_finish')
    def on_finish_with_instana(wrapped, instance, argv, kwargs):
        try:
            if not hasattr(instance.request, '_instana'):
                return wrapped(*argv, **kwargs)

            with instance.request._instana as scope:
                # Response header tracking support
                extract_custom_headers(scope.span, instance._headers)

                status_code = instance.get_status()

                # Mark 500 responses as errored
                if 500 <= status_code:
                    scope.span.mark_as_errored()

                scope.span.set_tag("http.status_code", status_code)

            return wrapped(*argv, **kwargs)
        except Exception:
            logger.debug("tornado on_finish", exc_info=True)


    @wrapt.patch_function_wrapper('tornado.web', 'RequestHandler.log_exception')
    def log_exception_with_instana(wrapped, instance, argv, kwargs):
        try:
            if not hasattr(instance.request, '_instana'):
                return wrapped(*argv, **kwargs)

            if not isinstance(argv[1], tornado.web.HTTPError):
                scope = instance.request._instana
                scope.span.log_exception(argv[0])

            return wrapped(*argv, **kwargs)
        except Exception:
            logger.debug("tornado log_exception", exc_info=True)


    logger.debug("Instrumenting tornado server")
except ImportError:
    pass



# (c) Copyright IBM Corp. 2024

import os
import json
import unittest

from instana.tracer import InstanaTracer
from instana.recorder import StanRecorder
from instana.agent.aws_eks_fargate import EKSFargateAgent
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer


class TestFargateCollector(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestFargateCollector, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None
        self.pwd = os.path.dirname(os.path.realpath(__file__))

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        os.environ["INSTANA_TRACER_ENVIRONMENT"] = "AWS_EKS_FARGATE"
        os.environ["INSTANA_ENDPOINT_URL"] = "https://localhost/notreal"
        os.environ["INSTANA_AGENT_KEY"] = "Fake_Key"

    def tearDown(self):
        """ Reset all environment variables of consequence """
        variable_names = (
                "INSTANA_TRACER_ENVIRONMENT",
                "AWS_EXECUTION_ENV", "INSTANA_EXTRA_HTTP_HEADERS",
                "INSTANA_ENDPOINT_URL", "INSTANA_ENDPOINT_PROXY",
                "INSTANA_AGENT_KEY", "INSTANA_ZONE", "INSTANA_TAGS"
                )

        for variable_name in variable_names:
            if variable_name in os.environ:
                os.environ.pop(variable_name)

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)

    def create_agent_and_setup_tracer(self):
        self.agent = EKSFargateAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

    def test_prepare_payload_basics(self):
        self.create_agent_and_setup_tracer()

        payload = self.agent.collector.prepare_payload()
        self.assertTrue(payload)

        self.assertEqual(2, len(payload.keys()))
        self.assertIn('spans',payload)
        self.assertIsInstance(payload['spans'], list)
        self.assertEqual(0, len(payload['spans']))
        self.assertIn('metrics', payload)
        self.assertEqual(1, len(payload['metrics'].keys()))
        self.assertIn('plugins', payload['metrics'])
        self.assertIsInstance(payload['metrics']['plugins'], list)
        self.assertEqual(2, len(payload['metrics']['plugins']))


        process_plugin = payload['metrics']['plugins'][0]
        #self.assertIn('data', process_plugin)

        runtime_plugin = payload['metrics']['plugins'][1]
        self.assertIn('name', runtime_plugin)
        self.assertIn('entityId', runtime_plugin)
        self.assertIn('data', runtime_plugin)

    def test_no_instana_zone(self):
        self.create_agent_and_setup_tracer()
        self.assertIsNone(self.agent.options.zone)




# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2017

import os

os.environ["INSTANA_TEST"] = "true"

if os.environ.get('GEVENT_STARLETTE_TEST'):
    from gevent import monkey
    monkey.patch_all()




# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import unittest

from moto import mock_aws
import boto3

from instana.singletons import tracer, agent
from ...helpers import get_first_span_by_filter

pwd = os.path.dirname(os.path.abspath(__file__))
upload_filename = os.path.abspath(pwd + '/../../data/boto3/test_upload_file.jpg')
download_target_filename = os.path.abspath(pwd + '/../../data/boto3/download_target_file.asdf')


class TestS3(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        self.mock = mock_aws()
        self.mock.start()
        self.s3 = boto3.client('s3', region_name='us-east-1')

    def tearDown(self):
        # Stop Moto after each test
        self.mock.stop()
        agent.options.allow_exit_as_root = False


    def test_vanilla_create_bucket(self):
        self.s3.create_bucket(Bucket="aws_bucket_name")

        result = self.s3.list_buckets()
        self.assertEqual(1, len(result['Buckets']))
        self.assertEqual(result['Buckets'][0]['Name'], 'aws_bucket_name')


    def test_s3_create_bucket(self):
        with tracer.start_active_span('test'):
            self.s3.create_bucket(Bucket="aws_bucket_name")

        result = self.s3.list_buckets()
        self.assertEqual(1, len(result['Buckets']))
        self.assertEqual(result['Buckets'][0]['Name'], 'aws_bucket_name')

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'CreateBucket')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'Bucket': 'aws_bucket_name'})
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/CreateBucket')


    def test_s3_create_bucket_as_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        self.s3.create_bucket(Bucket="aws_bucket_name")

        agent.options.allow_exit_as_root = False
        result = self.s3.list_buckets()
        self.assertEqual(1, len(result['Buckets']))
        self.assertEqual(result['Buckets'][0]['Name'], 'aws_bucket_name')

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))
        boto_span = spans[0]
        self.assertTrue(boto_span)
        self.assertEqual(boto_span.n, "boto3")
        self.assertIsNone(boto_span.p)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'CreateBucket')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'Bucket': 'aws_bucket_name'})
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/CreateBucket')


    def test_s3_list_buckets(self):
        with tracer.start_active_span('test'):
            result = self.s3.list_buckets()

        self.assertEqual(0, len(result['Buckets']))
        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'ListBuckets')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {})
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/ListBuckets')


    def test_s3_vanilla_upload_file(self):
        object_name = 'aws_key_name'
        bucket_name = 'aws_bucket_name'

        self.s3.create_bucket(Bucket=bucket_name)
        result = self.s3.upload_file(upload_filename, bucket_name, object_name)
        self.assertIsNone(result)


    def test_s3_upload_file(self):
        object_name = 'aws_key_name'
        bucket_name = 'aws_bucket_name'

        self.s3.create_bucket(Bucket=bucket_name)

        with tracer.start_active_span('test'):
            self.s3.upload_file(upload_filename, bucket_name, object_name)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'upload_file')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        payload = {'Filename': upload_filename, 'Bucket': 'aws_bucket_name', 'Key': 'aws_key_name'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/upload_file')


    def test_s3_upload_file_obj(self):
        object_name = 'aws_key_name'
        bucket_name = 'aws_bucket_name'

        self.s3.create_bucket(Bucket=bucket_name)

        with tracer.start_active_span('test'):
            with open(upload_filename, "rb") as fd:
                self.s3.upload_fileobj(fd, bucket_name, object_name)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'upload_fileobj')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        payload = {'Bucket': 'aws_bucket_name', 'Key': 'aws_key_name'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/upload_fileobj')


    def test_s3_download_file(self):
        object_name = 'aws_key_name'
        bucket_name = 'aws_bucket_name'

        self.s3.create_bucket(Bucket=bucket_name)
        self.s3.upload_file(upload_filename, bucket_name, object_name)

        with tracer.start_active_span('test'):
            self.s3.download_file(bucket_name, object_name, download_target_filename)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'download_file')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        payload = {'Bucket': 'aws_bucket_name', 'Key': 'aws_key_name', 'Filename': '%s' % download_target_filename}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/download_file')


    def test_s3_download_file_obj(self):
        object_name = 'aws_key_name'
        bucket_name = 'aws_bucket_name'

        self.s3.create_bucket(Bucket=bucket_name)
        self.s3.upload_file(upload_filename, bucket_name, object_name)

        with tracer.start_active_span('test'):
            with open(download_target_filename, "wb") as fd:
                self.s3.download_fileobj(bucket_name, object_name, fd)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'download_fileobj')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/download_fileobj')


    def test_request_header_capture_before_call(self):

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This', 'X-Capture-That']

        # Access the event system on the S3 client
        event_system = self.s3.meta.events

        request_headers = {
                'X-Capture-This': 'this',
                'X-Capture-That': 'that'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_call(params, **kwargs):
            params['headers'].update(request_headers)

        # Register the function to before-call event.
        event_system.register('before-call.s3.CreateBucket', add_custom_header_before_call)

        with tracer.start_active_span('test'):
            self.s3.create_bucket(Bucket="aws_bucket_name")

        result = self.s3.list_buckets()
        self.assertEqual(1, len(result['Buckets']))
        self.assertEqual(result['Buckets'][0]['Name'], 'aws_bucket_name')

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'CreateBucket')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'Bucket': 'aws_bucket_name'})
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/CreateBucket')

        self.assertIn("X-Capture-This", boto_span.data["http"]["header"])
        self.assertEqual("this", boto_span.data["http"]["header"]["X-Capture-This"])
        self.assertIn("X-Capture-That", boto_span.data["http"]["header"])
        self.assertEqual("that", boto_span.data["http"]["header"]["X-Capture-That"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_request_header_capture_before_sign(self):

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Custom-1', 'X-Custom-2']

        # Access the event system on the S3 client
        event_system = self.s3.meta.events

        request_headers = {
                'X-Custom-1': 'Value1',
                'X-Custom-2': 'Value2'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_sign(request, **kwargs):
            for name, value in request_headers.items():
                request.headers.add_header(name, value)

        # Register the function to before-sign event.
        event_system.register_first('before-sign.s3.CreateBucket', add_custom_header_before_sign)

        with tracer.start_active_span('test'):
            self.s3.create_bucket(Bucket="aws_bucket_name")

        result = self.s3.list_buckets()
        self.assertEqual(1, len(result['Buckets']))
        self.assertEqual(result['Buckets'][0]['Name'], 'aws_bucket_name')

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'CreateBucket')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'Bucket': 'aws_bucket_name'})
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/CreateBucket')

        self.assertIn("X-Custom-1", boto_span.data["http"]["header"])
        self.assertEqual("Value1", boto_span.data["http"]["header"]["X-Custom-1"])
        self.assertIn("X-Custom-2", boto_span.data["http"]["header"])
        self.assertEqual("Value2", boto_span.data["http"]["header"]["X-Custom-2"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_response_header_capture(self):

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This-Too', 'X-Capture-That-Too']

        # Access the event system on the S3 client
        event_system = self.s3.meta.events

        response_headers = {
            "X-Capture-This-Too": "this too",
            "X-Capture-That-Too": "that too",
        }

        # Create a function that sets the custom headers in the after-call event.
        def modify_after_call_args(parsed, **kwargs):
            parsed['ResponseMetadata']['HTTPHeaders'].update(response_headers)

        # Register the function to an event
        event_system.register('after-call.s3.CreateBucket', modify_after_call_args)

        with tracer.start_active_span('test'):
            self.s3.create_bucket(Bucket="aws_bucket_name")

        result = self.s3.list_buckets()
        self.assertEqual(1, len(result['Buckets']))
        self.assertEqual(result['Buckets'][0]['Name'], 'aws_bucket_name')

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'CreateBucket')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://s3.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'Bucket': 'aws_bucket_name'})
        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://s3.amazonaws.com:443/CreateBucket')

        self.assertIn("X-Capture-This-Too", boto_span.data["http"]["header"])
        self.assertEqual("this too", boto_span.data["http"]["header"]["X-Capture-This-Too"])
        self.assertIn("X-Capture-That-Too", boto_span.data["http"]["header"])
        self.assertEqual("that too", boto_span.data["http"]["header"]["X-Capture-That-Too"])

        agent.options.extra_http_headers = original_extra_http_headers



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import time
import unittest
import multiprocessing

import requests

from instana.singletons import async_tracer
from tests.apps.fastapi_app import launch_fastapi
from ..helpers import testenv
from ..helpers import get_first_span_by_filter


class TestFastAPI(unittest.TestCase):
    def setUp(self):
        self.proc = multiprocessing.Process(target=launch_fastapi, args=(), daemon=True)
        self.proc.start()
        time.sleep(2)

    def tearDown(self):
        # Kill server after tests
        self.proc.kill()

    def test_vanilla_get(self):
        result = requests.get(testenv["fastapi_server"] + "/")

        self.assertEqual(result.status_code, 200)
        self.assertIn("X-INSTANA-T", result.headers)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")
        self.assertIn("Server-Timing", result.headers)

        spans = async_tracer.recorder.queued_spans()
        # FastAPI instrumentation (like all instrumentation) _always_ traces unless told otherwise
        self.assertEqual(len(spans), 1)
        self.assertEqual(spans[0].n, "asgi")

    def test_basic_get(self):
        result = None
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/")

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])

    def test_400(self):
        result = None
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/400")

        self.assertEqual(result.status_code, 400)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/400")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/400")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 400)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])

    def test_500(self):
        result = None
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/500")

        self.assertEqual(result.status_code, 500)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertEqual(asgi_span.ec, 1)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/500")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/500")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 500)
        self.assertEqual(asgi_span.data["http"]["error"], "500 response")

        self.assertIsNone(asgi_span.data["http"]["params"])

    def test_path_templates(self):
        result = None
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/users/1")

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/users/1")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/users/{user_id}")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])

    def test_secret_scrubbing(self):
        result = None
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/?secret=shhh")

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertEqual(asgi_span.data["http"]["params"], "secret=<redacted>")

    def test_synthetic_request(self):
        request_headers = {"X-INSTANA-SYNTHETIC": "1"}
        with async_tracer.start_active_span("test"):
            result = requests.get(
                testenv["fastapi_server"] + "/", headers=request_headers
            )

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])

        self.assertTrue(asgi_span.sy)
        self.assertIsNone(urllib3_span.sy)
        self.assertIsNone(test_span.sy)

    def test_request_header_capture(self):
        from instana.singletons import agent

        # The background FastAPI server is pre-configured with custom headers to capture

        request_headers = {"X-Capture-This": "this", "X-Capture-That": "that"}

        with async_tracer.start_active_span("test"):
            result = requests.get(
                testenv["fastapi_server"] + "/", headers=request_headers
            )

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])

        self.assertIn("X-Capture-This", asgi_span.data["http"]["header"])
        self.assertEqual("this", asgi_span.data["http"]["header"]["X-Capture-This"])
        self.assertIn("X-Capture-That", asgi_span.data["http"]["header"])
        self.assertEqual("that", asgi_span.data["http"]["header"]["X-Capture-That"])

    def test_response_header_capture(self):
        from instana.singletons import agent

        # The background FastAPI server is pre-configured with custom headers to capture

        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/response_headers")

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/response_headers")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/response_headers")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])

        self.assertIn("X-Capture-This-Too", asgi_span.data["http"]["header"])
        self.assertEqual("this too", asgi_span.data["http"]["header"]["X-Capture-This-Too"])
        self.assertIn("X-Capture-That-Too", asgi_span.data["http"]["header"])
        self.assertEqual("that too", asgi_span.data["http"]["header"]["X-Capture-That-Too"])

    def test_non_async_simple(self):
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/non_async_simple")

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(5, len(spans))

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = (
            lambda span: span.n == "urllib3" and span.p == test_span.s
        )
        urllib3_span1 = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span1)

        span_filter = (
            lambda span: span.n == "asgi" and span.p == urllib3_span1.s
        )
        asgi_span1 = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span1)

        span_filter = (
            lambda span: span.n == "urllib3" and span.p == asgi_span1.s
        )
        urllib3_span2 = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span2)

        span_filter = (
            lambda span: span.n == "asgi" and span.p == urllib3_span2.s
        )
        asgi_span2 = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span2)

        # Same traceId
        traceId = test_span.t
        self.assertEqual(traceId, urllib3_span1.t)
        self.assertEqual(traceId, asgi_span1.t)
        self.assertEqual(traceId, urllib3_span2.t)
        self.assertEqual(traceId, asgi_span2.t)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span1.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span1.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span1.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span1.ec)
        self.assertEqual(asgi_span1.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span1.data["http"]["path"], "/non_async_simple")
        self.assertEqual(asgi_span1.data["http"]["path_tpl"], "/non_async_simple")
        self.assertEqual(asgi_span1.data["http"]["method"], "GET")
        self.assertEqual(asgi_span1.data["http"]["status"], 200)

        self.assertIsNone(asgi_span1.data["http"]["error"])
        self.assertIsNone(asgi_span1.data["http"]["params"])

    def test_non_async_threadpool(self):
        with async_tracer.start_active_span("test"):
            result = requests.get(testenv["fastapi_server"] + "/non_async_threadpool")

        self.assertEqual(result.status_code, 200)

        spans = async_tracer.recorder.queued_spans()
        self.assertEqual(3, len(spans))

        span_filter = (
            lambda span: span.n == "sdk" and span.data["sdk"]["name"] == "test"
        )
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == "asgi"
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        # Same traceId
        self.assertEqual(test_span.t, urllib3_span.t)
        self.assertEqual(urllib3_span.t, asgi_span.t)

        # Parent relationships
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)

        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)

        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], "1")

        self.assertIn("Server-Timing", result.headers)
        server_timing_value = "intid;desc=%s" % asgi_span.t
        self.assertEqual(result.headers["Server-Timing"], server_timing_value)

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data["http"]["host"], "127.0.0.1")
        self.assertEqual(asgi_span.data["http"]["path"], "/non_async_threadpool")
        self.assertEqual(asgi_span.data["http"]["path_tpl"], "/non_async_threadpool")
        self.assertEqual(asgi_span.data["http"]["method"], "GET")
        self.assertEqual(asgi_span.data["http"]["status"], 200)

        self.assertIsNone(asgi_span.data["http"]["error"])
        self.assertIsNone(asgi_span.data["http"]["params"])



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import importlib.util
import os
import sys
import pytest

if importlib.util.find_spec("celery"):
    pytest_plugins = ("celery.contrib.pytest",)

# Set our testing flags
os.environ["INSTANA_TEST"] = "true"
# os.environ["INSTANA_DEBUG"] = "true"

# Make sure the instana package is fully loaded
import instana

collect_ignore_glob = []

# Cassandra and gevent tests are run in dedicated jobs on CircleCI and will
# be run explicitly.  (So always exclude them here)
if not os.environ.get("CASSANDRA_TEST"):
    collect_ignore_glob.append("*test_cassandra*")

if not os.environ.get("COUCHBASE_TEST"):
    collect_ignore_glob.append("*test_couchbase*")

if not os.environ.get("GEVENT_STARLETTE_TEST"):
    collect_ignore_glob.append("*test_gevent*")
    collect_ignore_glob.append("*test_starlette*")

# Python 3.10 support is incomplete yet
# TODO: Remove this once we start supporting Tornado >= 6.0
if sys.version_info >= (3, 10):
    collect_ignore_glob.append("*test_tornado*")
    # Furthermore on Python 3.11 the above TC is skipped:
    # tests/opentracing/test_ot_span.py::TestOTSpan::test_stacks
    # TODO: Remove that once we find a workaround or DROP opentracing!

if sys.version_info >= (3, 11):
    if not os.environ.get("GOOGLE_CLOUD_TEST"):
        collect_ignore_glob.append("*test_google-cloud*")

if sys.version_info >= (3, 13):
    # TODO: Test Case failures for unknown reason:
    collect_ignore_glob.append("*test_aiohttp_server*")
    collect_ignore_glob.append("*test_celery*")

    # Currently there is a runtime incompatibility caused by the library:
    # `undefined symbol: _PyErr_WriteUnraisableMsg`
    collect_ignore_glob.append("*boto3*")

    # Currently there is a runtime incompatibility caused by the library:
    # `undefined symbol: _PyInterpreterState_Get`
    collect_ignore_glob.append("*test_psycopg2*")
    collect_ignore_glob.append("*test_sqlalchemy*")

    # Currently the latest version of pyramid depends on the `cgi` module
    # which has been deprecated since Python 3.11 and finally removed in 3.13
    # `ModuleNotFoundError: No module named 'cgi'`
    collect_ignore_glob.append("*test_pyramid*")

    # Currently not installable dependencies because of 3.13 incompatibilities
    collect_ignore_glob.append("*test_fastapi*")
    collect_ignore_glob.append("*test_google-cloud-pubsub*")
    collect_ignore_glob.append("*test_google-cloud-storage*")
    collect_ignore_glob.append("*test_grpcio*")
    collect_ignore_glob.append("*test_sanic*")


@pytest.fixture(scope="session")
def celery_config():
    return {
        "broker_connection_retry_on_startup": True,
        "broker_url": "redis://localhost:6379",
        "result_backend": "redis://localhost:6379",
    }


@pytest.fixture(scope="session")
def celery_enable_logging():
    return True


@pytest.fixture(scope="session")
def celery_includes():
    return {"tests.frameworks.test_celery"}



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import boto3
import unittest

from moto import mock_aws

from instana.singletons import tracer, agent
from ...helpers import get_first_span_by_filter

pwd = os.path.dirname(os.path.abspath(__file__))

class TestSes(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        self.mock = mock_aws()
        self.mock.start()
        self.ses = boto3.client('ses', region_name='us-east-1')

    def tearDown(self):
        # Stop Moto after each test
        self.mock.stop()


    def test_vanilla_verify_email(self):
        result = self.ses.verify_email_identity(EmailAddress='pglombardo+instana299@tuta.io')
        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)


    def test_verify_email(self):
        with tracer.start_active_span('test'):
            result = self.ses.verify_email_identity(EmailAddress='pglombardo+instana299@tuta.io')

        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'VerifyEmailIdentity')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://email.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'EmailAddress': 'pglombardo+instana299@tuta.io'})

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://email.us-east-1.amazonaws.com:443/VerifyEmailIdentity')


    def test_verify_email_as_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        result = self.ses.verify_email_identity(EmailAddress='pglombardo+instana299@tuta.io')

        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))
        boto_span = spans[0]
        self.assertTrue(boto_span)
        self.assertEqual(boto_span.n, "boto3")
        self.assertIsNone(boto_span.p)
        self.assertIsNone(boto_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'VerifyEmailIdentity')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://email.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'EmailAddress': 'pglombardo+instana299@tuta.io'})

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://email.us-east-1.amazonaws.com:443/VerifyEmailIdentity')


    def test_request_header_capture_before_call(self):

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This', 'X-Capture-That']

        # Access the event system on the S3 client
        event_system = self.ses.meta.events

        request_headers = {
                'X-Capture-This': 'this',
                'X-Capture-That': 'that'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_call(params, **kwargs):
            params['headers'].update(request_headers)

        # Register the function to before-call event.
        event_system.register('before-call.ses.VerifyEmailIdentity', add_custom_header_before_call)

        with tracer.start_active_span('test'):
            result = self.ses.verify_email_identity(EmailAddress='pglombardo+instana299@tuta.io')

        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'VerifyEmailIdentity')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://email.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'EmailAddress': 'pglombardo+instana299@tuta.io'})

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://email.us-east-1.amazonaws.com:443/VerifyEmailIdentity')

        self.assertIn("X-Capture-This", boto_span.data["http"]["header"])
        self.assertEqual("this", boto_span.data["http"]["header"]["X-Capture-This"])
        self.assertIn("X-Capture-That", boto_span.data["http"]["header"])
        self.assertEqual("that", boto_span.data["http"]["header"]["X-Capture-That"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_request_header_capture_before_sign(self):

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Custom-1', 'X-Custom-2']

        # Access the event system on the S3 client
        event_system = self.ses.meta.events

        request_headers = {
                'X-Custom-1': 'Value1',
                'X-Custom-2': 'Value2'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_sign(request, **kwargs):
            for name, value in request_headers.items():
                request.headers.add_header(name, value)

        # Register the function to before-sign event.
        event_system.register_first('before-sign.ses.VerifyEmailIdentity', add_custom_header_before_sign)

        with tracer.start_active_span('test'):
            result = self.ses.verify_email_identity(EmailAddress='pglombardo+instana299@tuta.io')

        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'VerifyEmailIdentity')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://email.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'EmailAddress': 'pglombardo+instana299@tuta.io'})

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://email.us-east-1.amazonaws.com:443/VerifyEmailIdentity')

        self.assertIn("X-Custom-1", boto_span.data["http"]["header"])
        self.assertEqual("Value1", boto_span.data["http"]["header"]["X-Custom-1"])
        self.assertIn("X-Custom-2", boto_span.data["http"]["header"])
        self.assertEqual("Value2", boto_span.data["http"]["header"]["X-Custom-2"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_response_header_capture(self):

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This-Too', 'X-Capture-That-Too']

        # Access the event system on the S3 client
        event_system = self.ses.meta.events

        response_headers = {
            "X-Capture-This-Too": "this too",
            "X-Capture-That-Too": "that too",
        }

        # Create a function that sets the custom headers in the after-call event.
        def modify_after_call_args(parsed, **kwargs):
            parsed['ResponseMetadata']['HTTPHeaders'].update(response_headers)

        # Register the function to an event
        event_system.register('after-call.ses.VerifyEmailIdentity', modify_after_call_args)

        with tracer.start_active_span('test'):
            result = self.ses.verify_email_identity(EmailAddress='pglombardo+instana299@tuta.io')

        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'VerifyEmailIdentity')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://email.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')
        self.assertDictEqual(boto_span.data['boto3']['payload'], {'EmailAddress': 'pglombardo+instana299@tuta.io'})

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://email.us-east-1.amazonaws.com:443/VerifyEmailIdentity')

        self.assertIn("X-Capture-This-Too", boto_span.data["http"]["header"])
        self.assertEqual("this too", boto_span.data["http"]["header"]["X-Capture-This-Too"])
        self.assertIn("X-Capture-That-Too", boto_span.data["http"]["header"])
        self.assertEqual("that too", boto_span.data["http"]["header"]["X-Capture-That-Too"])

        agent.options.extra_http_headers = original_extra_http_headers



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2017

import unittest
import instana


class TestIdManagement(unittest.TestCase):
    def test_id_generation(self):
        count = 0
        while count <= 10000:
            id = instana.util.ids.generate_id()
            base10_id = int(id, 16)
            self.assertGreaterEqual(base10_id, 0)
            self.assertLessEqual(base10_id, 18446744073709551615)
            count += 1


    def test_various_header_to_id_conversion(self):
        # Get a hex string to test against & convert
        header_id = instana.util.ids.generate_id()
        converted_id = instana.util.ids.header_to_long_id(header_id)
        self.assertEqual(header_id, converted_id)

        # Hex value - result should be left padded
        result = instana.util.ids.header_to_long_id('abcdef')
        self.assertEqual('0000000000abcdef', result)

        # Hex value
        result = instana.util.ids.header_to_long_id('0123456789abcdef')
        self.assertEqual('0123456789abcdef', result)

        # Very long incoming header should just return the rightmost 16 bytes
        result = instana.util.ids.header_to_long_id('0x0123456789abcdef0123456789abcdef')
        self.assertEqual('0x0123456789abcdef0123456789abcdef', result)


    def test_header_to_id_conversion_with_bogus_header(self):
        # Bogus nil arg
        bogus_result = instana.util.ids.header_to_long_id(None)
        self.assertEqual(instana.util.ids.BAD_ID, bogus_result)

        # Bogus Integer arg
        bogus_result = instana.util.ids.header_to_long_id(1234)
        self.assertEqual(instana.util.ids.BAD_ID, bogus_result)

        # Bogus Array arg
        bogus_result = instana.util.ids.header_to_long_id([1234])
        self.assertEqual(instana.util.ids.BAD_ID, bogus_result)

        # Bogus Hex Values in String
        bogus_result = instana.util.ids.header_to_long_id('0xZZZZZZ')
        self.assertEqual(instana.util.ids.BAD_ID, bogus_result)

        bogus_result = instana.util.ids.header_to_long_id('ZZZZZZ')
        self.assertEqual(instana.util.ids.BAD_ID, bogus_result)



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import re
import sys
import json
import time
import unittest
from uuid import UUID

import opentracing

from instana.util import to_json
from instana.singletons import agent, tracer
from ..helpers import get_first_span_by_filter


class TestOTSpan(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        agent.options.service_name = None
        opentracing.tracer = tracer
        recorder = opentracing.tracer.recorder
        recorder.clear_spans()

    def tearDown(self):
        """ Do nothing for now """
        return None

    def test_span_interface(self):
        span = opentracing.tracer.start_span("blah")
        self.assertTrue(hasattr(span, "finish"))
        self.assertTrue(hasattr(span, "set_tag"))
        self.assertTrue(hasattr(span, "tags"))
        self.assertTrue(hasattr(span, "operation_name"))
        self.assertTrue(hasattr(span, "set_baggage_item"))
        self.assertTrue(hasattr(span, "get_baggage_item"))
        self.assertTrue(hasattr(span, "context"))
        self.assertTrue(hasattr(span, "log"))

    def test_span_ids(self):
        count = 0
        while count <= 1000:
            count += 1
            span = opentracing.tracer.start_span("test_span_ids")
            context = span.context
            self.assertTrue(0 <= int(context.span_id, 16) <= 18446744073709551615)
            self.assertTrue(0 <= int(context.trace_id, 16) <= 18446744073709551615)

    # Python 3.11 support is incomplete yet
    # TODO: Remove this once we find a workaround or DROP opentracing!
    @unittest.skipIf(sys.version_info >= (3, 11), reason="Raises not Implemented exception in OSX")
    def test_stacks(self):
        # Entry spans have no stack attached by default
        wsgi_span = opentracing.tracer.start_span("wsgi")
        self.assertIsNone(wsgi_span.stack)

        # SDK spans have no stack attached by default
        sdk_span = opentracing.tracer.start_span("unregistered_span_type")
        self.assertIsNone(sdk_span.stack)

        # Exit spans are no longer than 30 frames
        exit_span = opentracing.tracer.start_span("urllib3")
        self.assertLessEqual(len(exit_span.stack), 30)

    def test_span_fields(self):
        span = opentracing.tracer.start_span("mycustom")
        self.assertEqual("mycustom", span.operation_name)
        self.assertTrue(span.context)

        span.set_tag("tagone", "string")
        span.set_tag("tagtwo", 150)

        self.assertEqual("string", span.tags['tagone'])
        self.assertEqual(150, span.tags['tagtwo'])

    @unittest.skipIf(sys.platform == "darwin", reason="Raises not Implemented exception in OSX")
    def test_span_queueing(self):
        recorder = opentracing.tracer.recorder

        count = 1
        while count <= 20:
            count += 1
            span = opentracing.tracer.start_span("queuethisplz")
            span.set_tag("tagone", "string")
            span.set_tag("tagtwo", 150)
            span.finish()

        self.assertEqual(20, recorder.queue_size())

    def test_sdk_spans(self):
        recorder = opentracing.tracer.recorder

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag("tagone", "string")
        span.set_tag("tagtwo", 150)
        span.set_tag('span.kind', "entry")
        time.sleep(0.5)
        span.finish()

        spans = recorder.queued_spans()
        self.assertEqual(1, len(spans))

        sdk_span = spans[0]
        self.assertEqual('sdk', sdk_span.n)
        self.assertEqual(None, sdk_span.p)
        self.assertEqual(sdk_span.s, sdk_span.t)
        self.assertTrue(sdk_span.ts)
        self.assertGreater(sdk_span.ts, 0)
        self.assertTrue(sdk_span.d)
        self.assertGreater(sdk_span.d, 0)

        self.assertTrue(sdk_span.data)
        self.assertTrue(sdk_span.data["sdk"])
        self.assertEqual('entry', sdk_span.data["sdk"]["type"])
        self.assertEqual('custom_sdk_span', sdk_span.data["sdk"]["name"])
        self.assertTrue(sdk_span.data["sdk"]["custom"])
        self.assertTrue(sdk_span.data["sdk"]["custom"]["tags"])

    def test_span_kind(self):
        recorder = opentracing.tracer.recorder

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "consumer")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "server")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "producer")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "client")
        span.finish()

        span = opentracing.tracer.start_span("custom_sdk_span")
        span.set_tag('span.kind', "blah")
        span.finish()

        spans = recorder.queued_spans()
        self.assertEqual(5, len(spans))

        span = spans[0]
        self.assertEqual('entry', span.data["sdk"]["type"])

        span = spans[1]
        self.assertEqual('entry', span.data["sdk"]["type"])

        span = spans[2]
        self.assertEqual('exit', span.data["sdk"]["type"])

        span = spans[3]
        self.assertEqual('exit', span.data["sdk"]["type"])

        span = spans[4]
        self.assertEqual('intermediate', span.data["sdk"]["type"])

        span = spans[0]
        self.assertEqual(1, span.k)

        span = spans[1]
        self.assertEqual(1, span.k)

        span = spans[2]
        self.assertEqual(2, span.k)

        span = spans[3]
        self.assertEqual(2, span.k)

        span = spans[4]
        self.assertEqual(3, span.k)

    def test_tag_values(self):
        with tracer.start_active_span('test') as scope:
            # Set a UUID class as a tag
            # If unchecked, this causes a json.dumps error: "ValueError: Circular reference detected"
            scope.span.set_tag('uuid', UUID(bytes=b'\x12\x34\x56\x78'*4))
            # Arbitrarily setting an instance of some class
            scope.span.set_tag('tracer', tracer)
            scope.span.set_tag('none', None)
            scope.span.set_tag('mylist', [1, 2, 3])
            scope.span.set_tag('myset', {"one", 2})

        spans = tracer.recorder.queued_spans()
        self.assertEqual(1, len(spans))

        test_span = spans[0]
        self.assertTrue(test_span)
        self.assertEqual(len(test_span.data['sdk']['custom']['tags']), 5)
        self.assertEqual(test_span.data['sdk']['custom']['tags']['uuid'], "UUID('12345678-1234-5678-1234-567812345678')")
        self.assertTrue(test_span.data['sdk']['custom']['tags']['tracer'])
        self.assertEqual(test_span.data['sdk']['custom']['tags']['none'], 'None')
        self.assertListEqual(test_span.data['sdk']['custom']['tags']['mylist'], [1, 2, 3])
        self.assertRegex(test_span.data['sdk']['custom']['tags']['myset'], r"\{.*,.*\}")

        # Convert to JSON
        json_data = to_json(test_span)
        self.assertTrue(json_data)

        # And back
        span_dict = json.loads(json_data)
        self.assertEqual(len(span_dict['data']['sdk']['custom']['tags']), 5)
        self.assertEqual(span_dict['data']['sdk']['custom']['tags']['uuid'], "UUID('12345678-1234-5678-1234-567812345678')")
        self.assertTrue(span_dict['data']['sdk']['custom']['tags']['tracer'])
        self.assertEqual(span_dict['data']['sdk']['custom']['tags']['none'], 'None')
        self.assertListEqual(span_dict['data']['sdk']['custom']['tags']['mylist'], [1, 2, 3])
        self.assertRegex(test_span.data['sdk']['custom']['tags']['myset'], r"{.*,.*}")

    def test_tag_names(self):
        with tracer.start_active_span('test') as scope:
            # Tag names (keys) must be strings
            scope.span.set_tag(1234567890, 'This should not get set')
            # Unicode key name
            scope.span.set_tag(u'asdf', 'This should be ok')

        spans = tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 1)

        test_span = spans[0]
        self.assertTrue(test_span)
        self.assertEqual(len(test_span.data['sdk']['custom']['tags']), 1)
        self.assertEqual(test_span.data['sdk']['custom']['tags']['asdf'], 'This should be ok')

        json_data = to_json(test_span)
        self.assertTrue(json_data)

    def test_custom_service_name(self):
        # Set a custom service name
        agent.options.service_name = "custom_service_name"

        with tracer.start_active_span('entry_span') as scope:
            scope.span.set_tag('span.kind', 'server')
            scope.span.set_tag(u'type', 'entry_span')

            with tracer.start_active_span('intermediate_span', child_of=scope.span) as exit_scope:
                exit_scope.span.set_tag(u'type', 'intermediate_span')

            with tracer.start_active_span('exit_span', child_of=scope.span) as exit_scope:
                exit_scope.span.set_tag('span.kind', 'client')
                exit_scope.span.set_tag(u'type', 'exit_span')

        spans = tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == "entry_span"
        entry_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(entry_span)

        filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == "intermediate_span"
        intermediate_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(intermediate_span)

        filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == "exit_span"
        exit_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(exit_span)

        self.assertTrue(entry_span)
        self.assertEqual(len(entry_span.data['sdk']['custom']['tags']), 2)
        self.assertEqual(entry_span.data['sdk']['custom']['tags']['type'], 'entry_span')
        self.assertEqual(entry_span.data['service'], 'custom_service_name')
        self.assertEqual(entry_span.k, 1)

        self.assertTrue(intermediate_span)
        self.assertEqual(len(intermediate_span.data['sdk']['custom']['tags']), 1)
        self.assertEqual(intermediate_span.data['sdk']['custom']['tags']['type'], 'intermediate_span')
        self.assertEqual(intermediate_span.data['service'], 'custom_service_name')
        self.assertEqual(intermediate_span.k, 3)

        self.assertTrue(exit_span)
        self.assertEqual(len(exit_span.data['sdk']['custom']['tags']), 2)
        self.assertEqual(exit_span.data['sdk']['custom']['tags']['type'], 'exit_span')
        self.assertEqual(exit_span.data['service'], 'custom_service_name')
        self.assertEqual(exit_span.k, 2)

    def test_span_log(self):
        with tracer.start_active_span('mylogspan') as scope:
            scope.span.log_kv({'Don McLean': 'American Pie'})
            scope.span.log_kv({'Elton John': 'Your Song'})

        spans = tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 1)

        my_log_span = spans[0]
        self.assertEqual(my_log_span.n, 'sdk')

        log_data = my_log_span.data['sdk']['custom']['logs']
        self.assertEqual(len(log_data), 2)



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2019


import wrapt
from opentracing.scope_managers.constants import ACTIVE_ATTR
from opentracing.scope_managers.contextvars import no_parent_scope

from ..configurator import config
from ..log import logger
from ..singletons import async_tracer

try:
    import asyncio

    @wrapt.patch_function_wrapper("asyncio", "ensure_future")
    def ensure_future_with_instana(wrapped, instance, argv, kwargs):
        if config["asyncio_task_context_propagation"]["enabled"] is False:
            with no_parent_scope():
                return wrapped(*argv, **kwargs)

        scope = async_tracer.scope_manager.active
        task = wrapped(*argv, **kwargs)

        if scope is not None:
            setattr(task, ACTIVE_ATTR, scope)

        return task

    if hasattr(asyncio, "create_task"):

        @wrapt.patch_function_wrapper("asyncio", "create_task")
        def create_task_with_instana(wrapped, instance, argv, kwargs):
            if config["asyncio_task_context_propagation"]["enabled"] is False:
                with no_parent_scope():
                    return wrapped(*argv, **kwargs)

            scope = async_tracer.scope_manager.active
            task = wrapped(*argv, **kwargs)

            if scope is not None:
                setattr(task, ACTIVE_ATTR, scope)

            return task

    logger.debug("Instrumenting asyncio")
except ImportError:
    pass



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

class _TraceContextMixin:
    def assertTraceContextPropagated(self, parent_span, child_span):
        self.assertEqual(parent_span.t, child_span.t)
        self.assertEqual(parent_span.s, child_span.p)
        self.assertNotEqual(parent_span.s, child_span.s)

    def assertErrorLogging(self, spans):
        for span in spans:
            self.assertIsNone(span.ec)



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import multiprocessing
import time
import pytest
import requests
import unittest

from ..helpers import testenv
from instana.singletons import tracer
from ..helpers import get_first_span_by_filter


class TestStarlette(unittest.TestCase):
    def setUp(self):
        from tests.apps.starlette_app import launch_starlette
        self.proc = multiprocessing.Process(target=launch_starlette, args=(), daemon=True)
        self.proc.start()
        time.sleep(2)

    def tearDown(self):
        self.proc.kill() # Kill server after tests

    def test_vanilla_get(self):
        result = requests.get(testenv["starlette_server"] + '/')
        self.assertTrue(result)
        spans = tracer.recorder.queued_spans()
        # Starlette instrumentation (like all instrumentation) _always_ traces unless told otherwise
        self.assertEqual(len(spans), 1)
        self.assertEqual(spans[0].n, 'asgi')

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], '1')
        self.assertIn("Server-Timing", result.headers)

    def test_basic_get(self):
        result = None
        with tracer.start_active_span('test'):
            result = requests.get(testenv["starlette_server"] + '/')

        self.assertTrue(result)

        spans = tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == 'test'
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == 'asgi'
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        self.assertTrue(test_span.t == urllib3_span.t == asgi_span.t)
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], '1')
        self.assertIn("Server-Timing", result.headers)
        self.assertEqual(result.headers["Server-Timing"], ("intid;desc=%s" % asgi_span.t))

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data['http']['host'], '127.0.0.1')
        self.assertEqual(asgi_span.data['http']['path'], '/')
        self.assertEqual(asgi_span.data['http']['path_tpl'], '/')
        self.assertEqual(asgi_span.data['http']['method'], 'GET')
        self.assertEqual(asgi_span.data['http']['status'], 200)
        self.assertIsNone(asgi_span.data['http']['error'])
        self.assertIsNone(asgi_span.data['http']['params'])

    def test_path_templates(self):
        result = None
        with tracer.start_active_span('test'):
            result = requests.get(testenv["starlette_server"] + '/users/1')

        self.assertTrue(result)

        spans = tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == 'test'
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == 'asgi'
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        self.assertTrue(test_span.t == urllib3_span.t == asgi_span.t)
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual( result.headers["X-INSTANA-S"], asgi_span.s)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], '1')
        self.assertIn("Server-Timing", result.headers)
        self.assertEqual(result.headers["Server-Timing"], ("intid;desc=%s" % asgi_span.t))

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data['http']['host'], '127.0.0.1')
        self.assertEqual(asgi_span.data['http']['path'], '/users/1')
        self.assertEqual(asgi_span.data['http']['path_tpl'], '/users/{user_id}')
        self.assertEqual(asgi_span.data['http']['method'], 'GET')
        self.assertEqual(asgi_span.data['http']['status'], 200)
        self.assertIsNone(asgi_span.data['http']['error'])
        self.assertIsNone(asgi_span.data['http']['params'])

    def test_secret_scrubbing(self):
        result = None
        with tracer.start_active_span('test'):
            result = requests.get(testenv["starlette_server"] + '/?secret=shhh')

        self.assertTrue(result)

        spans = tracer.recorder.queued_spans()
        assert len(spans) == 3

        span_filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == 'test'
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == 'asgi'
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        self.assertTrue(test_span.t == urllib3_span.t == asgi_span.t)
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], '1')
        self.assertIn("Server-Timing", result.headers)
        self.assertEqual(result.headers["Server-Timing"], ("intid;desc=%s" % asgi_span.t))

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data['http']['host'], '127.0.0.1')
        self.assertEqual(asgi_span.data['http']['path'], '/')
        self.assertEqual(asgi_span.data['http']['path_tpl'], '/')
        self.assertEqual(asgi_span.data['http']['method'], 'GET')
        self.assertEqual(asgi_span.data['http']['status'], 200)
        self.assertIsNone(asgi_span.data['http']['error'])
        self.assertEqual(asgi_span.data['http']['params'], 'secret=<redacted>')

    def test_synthetic_request(self):
        request_headers = {
            'X-INSTANA-SYNTHETIC': '1'
        }
        with tracer.start_active_span('test'):
            result = requests.get(testenv["starlette_server"] + '/', headers=request_headers)

        self.assertTrue(result)

        spans = tracer.recorder.queued_spans()
        assert len(spans) == 3

        span_filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == 'test'
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == 'asgi'
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        self.assertTrue(test_span.t == urllib3_span.t == asgi_span.t)
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual(result.headers["X-INSTANA-S"], asgi_span.s)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual(result.headers["X-INSTANA-L"], '1')
        self.assertIn("Server-Timing", result.headers)
        self.assertEqual(result.headers["Server-Timing"], ("intid;desc=%s" % asgi_span.t))

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data['http']['host'], '127.0.0.1')
        self.assertEqual(asgi_span.data['http']['path'], '/')
        self.assertEqual(asgi_span.data['http']['path_tpl'], '/')
        self.assertEqual(asgi_span.data['http']['method'], 'GET')
        self.assertEqual(asgi_span.data['http']['status'], 200)
        self.assertIsNone(asgi_span.data['http']['error'])
        self.assertIsNone(asgi_span.data['http']['params'])

        self.assertTrue(asgi_span.sy)
        self.assertIsNone(urllib3_span.sy)
        self.assertIsNone(test_span.sy)

    def test_custom_header_capture(self):
        from instana.singletons import agent

        # The background Starlette server is pre-configured with custom headers to capture

        request_headers = {
            'X-Capture-This': 'this',
            'X-Capture-That': 'that'
        }
        with tracer.start_active_span('test'):
            result = requests.get(testenv["starlette_server"] + '/', headers=request_headers)

        self.assertTrue(result)

        spans = tracer.recorder.queued_spans()
        self.assertEqual(len(spans), 3)

        span_filter = lambda span: span.n == "sdk" and span.data['sdk']['name'] == 'test'
        test_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(test_span)

        span_filter = lambda span: span.n == "urllib3"
        urllib3_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(urllib3_span)

        span_filter = lambda span: span.n == 'asgi'
        asgi_span = get_first_span_by_filter(spans, span_filter)
        self.assertTrue(asgi_span)

        self.assertTrue(test_span.t == urllib3_span.t == asgi_span.t)
        self.assertEqual(asgi_span.p, urllib3_span.s)
        self.assertEqual(urllib3_span.p, test_span.s)

        self.assertIn("X-INSTANA-T", result.headers)
        self.assertEqual(result.headers["X-INSTANA-T"], asgi_span.t)
        self.assertIn("X-INSTANA-S", result.headers)
        self.assertEqual( result.headers["X-INSTANA-S"], asgi_span.s)
        self.assertIn("X-INSTANA-L", result.headers)
        self.assertEqual( result.headers["X-INSTANA-L"], '1')
        self.assertIn("Server-Timing", result.headers)
        self.assertEqual(result.headers["Server-Timing"], ("intid;desc=%s" % asgi_span.t))

        self.assertIsNone(asgi_span.ec)
        self.assertEqual(asgi_span.data['http']['host'], '127.0.0.1')
        self.assertEqual(asgi_span.data['http']['path'], '/')
        self.assertEqual(asgi_span.data['http']['path_tpl'], '/')
        self.assertEqual(asgi_span.data['http']['method'], 'GET')
        self.assertEqual(asgi_span.data['http']['status'], 200)
        self.assertIsNone(asgi_span.data['http']['error'])
        self.assertIsNone(asgi_span.data['http']['params'])

        self.assertIn("X-Capture-This", asgi_span.data["http"]["header"])
        self.assertEqual("this", asgi_span.data["http"]["header"]["X-Capture-This"])
        self.assertIn("X-Capture-That", asgi_span.data["http"]["header"])
        self.assertEqual("that", asgi_span.data["http"]["header"]["X-Capture-That"])



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import json
import unittest
import logging

from ..helpers import testenv
from instana.singletons import agent, tracer

import pymongo
import bson

logger = logging.getLogger(__name__)

pymongoversion = unittest.skipIf(
    pymongo.version_tuple >= (4, 0), reason="map reduce is removed in pymongo 4.0"
)


class TestPyMongoTracer(unittest.TestCase):
    def setUp(self):
        self.client = pymongo.MongoClient(host=testenv['mongodb_host'], port=int(testenv['mongodb_port']),
                                          username=testenv['mongodb_user'], password=testenv['mongodb_pw'])
        self.client.test.records.delete_many(filter={})

        self.recorder = tracer.recorder
        self.recorder.clear_spans()

    def tearDown(self):
        self.client.close()
        agent.options.allow_exit_as_root = False

    def test_successful_find_query(self):
        with tracer.start_active_span("test"):
            self.client.test.records.find_one({"type": "string"})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 2)

        db_span = spans[0]
        test_span = spans[1]

        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"], "find")

        self.assertEqual(db_span.data["mongo"]["filter"], '{"type": "string"}')
        self.assertIsNone(db_span.data["mongo"]["json"])

    def test_successful_find_query_as_root_span(self):
        agent.options.allow_exit_as_root = True
        self.client.test.records.find_one({"type": "string"})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 1)

        db_span = spans[0]

        self.assertEqual(db_span.p, None)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"], "find")

        self.assertEqual(db_span.data["mongo"]["filter"], '{"type": "string"}')
        self.assertIsNone(db_span.data["mongo"]["json"])

    def test_successful_insert_query(self):
        with tracer.start_active_span("test"):
            self.client.test.records.insert_one({"type": "string"})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 2)

        db_span = spans[0]
        test_span = spans[1]

        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"], "insert")

        self.assertIsNone(db_span.data["mongo"]["filter"])

    def test_successful_update_query(self):
        with tracer.start_active_span("test"):
            self.client.test.records.update_one({"type": "string"}, {"$set": {"type": "int"}})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 2)

        db_span = spans[0]
        test_span = spans[1]

        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"], "update")

        self.assertIsNone(db_span.data["mongo"]["filter"])
        self.assertIsNotNone(db_span.data["mongo"]["json"])

        payload = json.loads(db_span.data["mongo"]["json"])
        self.assertIn({
                        "q": {"type": "string"},
                        "u": {"$set": {"type": "int"}},
                        "multi": False,
                        "upsert": False
                       }, payload)

    def test_successful_delete_query(self):
        with tracer.start_active_span("test"):
            self.client.test.records.delete_one(filter={"type": "string"})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 2)

        db_span = spans[0]
        test_span = spans[1]

        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"], "delete")

        self.assertIsNone(db_span.data["mongo"]["filter"])
        self.assertIsNotNone(db_span.data["mongo"]["json"])

        payload = json.loads(db_span.data["mongo"]["json"])
        self.assertIn({"q": {"type": "string"}, "limit": 1}, payload)

    def test_successful_aggregate_query(self):
        with tracer.start_active_span("test"):
            self.client.test.records.count_documents({"type": "string"})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 2)

        db_span = spans[0]
        test_span = spans[1]

        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"], "aggregate")

        self.assertIsNone(db_span.data["mongo"]["filter"])
        self.assertIsNotNone(db_span.data["mongo"]["json"])

        payload = json.loads(db_span.data["mongo"]["json"])
        self.assertIn({"$match": {"type": "string"}}, payload)

    @pymongoversion
    def test_successful_map_reduce_query(self):
        mapper = "function () { this.tags.forEach(function(z) { emit(z, 1); }); }"
        reducer = "function (key, values) { return len(values); }"

        with tracer.start_active_span("test"):
            self.client.test.records.map_reduce(bson.code.Code(mapper), bson.code.Code(reducer), "results",
                                              query={"x": {"$lt": 2}})

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 2)

        db_span = spans[0]
        test_span = spans[1]

        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "mongo")
        self.assertEqual(db_span.data["mongo"]["service"], "%s:%s" % (testenv['mongodb_host'], testenv['mongodb_port']))
        self.assertEqual(db_span.data["mongo"]["namespace"], "test.records")
        self.assertEqual(db_span.data["mongo"]["command"].lower(),
                         "mapreduce")  # mapreduce command was renamed to mapReduce in pymongo 3.9.0

        self.assertEqual(db_span.data["mongo"]["filter"], '{"x": {"$lt": 2}}')
        self.assertIsNotNone(db_span.data["mongo"]["json"])

        payload = json.loads(db_span.data["mongo"]["json"])
        self.assertEqual(payload["map"], {"$code": mapper}, db_span.data["mongo"]["json"])
        self.assertEqual(payload["reduce"], {"$code": reducer}, db_span.data["mongo"]["json"])

    def test_successful_mutiple_queries(self):
        with tracer.start_active_span("test"):
            self.client.test.records.bulk_write([pymongo.InsertOne({"type": "string"}),
                                                 pymongo.UpdateOne({"type": "string"}, {"$set": {"type": "int"}}),
                                                 pymongo.DeleteOne({"type": "string"})])

        self.assertIsNone(tracer.active_span)

        spans = self.recorder.queued_spans()
        self.assertEqual(len(spans), 4)

        test_span = spans.pop()

        seen_span_ids = set()
        commands = []
        for span in spans:
            self.assertEqual(test_span.t, span.t)
            self.assertEqual(span.p, test_span.s)

            # check if all spans got a unique id
            self.assertNotIn(span.s, seen_span_ids)

            seen_span_ids.add(span.s)
            commands.append(span.data["mongo"]["command"])

        # ensure spans are ordered the same way as commands
        self.assertListEqual(commands, ["insert", "update", "delete"])




# (c) Copyright IBM Corp. 2024

import os
import logging
import unittest

from instana.tracer import InstanaTracer
from instana.options import EKSFargateOptions
from instana.recorder import StanRecorder
from instana.agent.aws_eks_fargate import EKSFargateAgent
from instana.singletons import get_agent, set_agent, get_tracer, set_tracer


class TestFargate(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestFargate, self).__init__(methodName)
        self.agent = None
        self.span_recorder = None
        self.tracer = None

        self.original_agent = get_agent()
        self.original_tracer = get_tracer()

    def setUp(self):
        os.environ["INSTANA_TRACER_ENVIRONMENT"] = "AWS_EKS_FARGATE"
        os.environ["INSTANA_ENDPOINT_URL"] = "https://localhost/notreal"
        os.environ["INSTANA_AGENT_KEY"] = "Fake_Key"

    def tearDown(self):
        """ Reset all environment variables of consequence """
        variable_names = (
                "INSTANA_TRACER_ENVIRONMENT",
                "AWS_EXECUTION_ENV", "INSTANA_EXTRA_HTTP_HEADERS",
                "INSTANA_ENDPOINT_URL", "INSTANA_ENDPOINT_PROXY",
                "INSTANA_AGENT_KEY", "INSTANA_LOG_LEVEL",
                "INSTANA_SECRETS", "INSTANA_DEBUG", "INSTANA_TAGS"
                )

        for variable_name in variable_names:
            if variable_name in os.environ:
                os.environ.pop(variable_name)

        set_agent(self.original_agent)
        set_tracer(self.original_tracer)

    def create_agent_and_setup_tracer(self):
        self.agent = EKSFargateAgent()
        self.span_recorder = StanRecorder(self.agent)
        self.tracer = InstanaTracer(recorder=self.span_recorder)
        set_agent(self.agent)
        set_tracer(self.tracer)

    def test_has_options(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent, 'options'))
        self.assertTrue(isinstance(self.agent.options, EKSFargateOptions))

    def test_missing_variables(self):
        with self.assertLogs("instana", level=logging.WARN) as context:
            os.environ.pop("INSTANA_ENDPOINT_URL")
            agent = EKSFargateAgent()
            self.assertFalse(agent.can_send())
            self.assertIsNone(agent.collector)
        self.assertIn('environment variables not set', context.output[0])

        os.environ["INSTANA_ENDPOINT_URL"] = "https://localhost/notreal"
        with self.assertLogs("instana", level=logging.WARN) as context:
            os.environ.pop("INSTANA_AGENT_KEY")
            agent = EKSFargateAgent()
            self.assertFalse(agent.can_send())
            self.assertIsNone(agent.collector)
        self.assertIn('environment variables not set', context.output[0])

    def test_default_secrets(self):
        self.create_agent_and_setup_tracer()
        self.assertIsNone(self.agent.options.secrets)
        self.assertTrue(hasattr(self.agent.options, 'secrets_matcher'))
        self.assertEqual(self.agent.options.secrets_matcher, 'contains-ignore-case')
        self.assertTrue(hasattr(self.agent.options, 'secrets_list'))
        self.assertListEqual(self.agent.options.secrets_list, ['key', 'pass', 'secret'])

    def test_custom_secrets(self):
        os.environ["INSTANA_SECRETS"] = "equals:love,war,games"
        self.create_agent_and_setup_tracer()

        self.assertTrue(hasattr(self.agent.options, 'secrets_matcher'))
        self.assertEqual(self.agent.options.secrets_matcher, 'equals')
        self.assertTrue(hasattr(self.agent.options, 'secrets_list'))
        self.assertListEqual(self.agent.options.secrets_list, ['love', 'war', 'games'])

    def test_default_tags(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent.options, 'tags'))
        self.assertIsNone(self.agent.options.tags)

    def test_has_extra_http_headers(self):
        self.create_agent_and_setup_tracer()
        self.assertTrue(hasattr(self.agent, 'options'))
        self.assertTrue(hasattr(self.agent.options, 'extra_http_headers'))

    def test_agent_extra_http_headers(self):
        os.environ['INSTANA_EXTRA_HTTP_HEADERS'] = "X-Test-Header;X-Another-Header;X-And-Another-Header"
        self.create_agent_and_setup_tracer()
        self.assertIsNotNone(self.agent.options.extra_http_headers)
        should_headers = ['x-test-header', 'x-another-header', 'x-and-another-header']
        self.assertListEqual(should_headers, self.agent.options.extra_http_headers)

    def test_agent_default_log_level(self):
        self.create_agent_and_setup_tracer()
        self.assertEqual(self.agent.options.log_level, logging.WARNING)

    def test_agent_custom_log_level(self):
        os.environ['INSTANA_LOG_LEVEL'] = "eRror"
        self.create_agent_and_setup_tracer()
        self.assertEqual(self.agent.options.log_level, logging.ERROR)

    def test_custom_proxy(self):
        os.environ["INSTANA_ENDPOINT_PROXY"] = "http://myproxy.123"
        self.create_agent_and_setup_tracer()
        self.assertDictEqual(self.agent.options.endpoint_proxy, {'https': "http://myproxy.123"})



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import logging
import unittest

from ..helpers import testenv
from instana.singletons import agent, tracer

import psycopg2
import psycopg2.extras
import psycopg2.extensions as ext

logger = logging.getLogger(__name__)


class TestPsycoPG2(unittest.TestCase):
    def setUp(self):
        deprecated_param_name = self.shortDescription() == 'test_deprecated_parameter_database'
        kwargs = {
            'host': testenv['postgresql_host'],
            'port': testenv['postgresql_port'],
            'user': testenv['postgresql_user'],
            'password': testenv['postgresql_pw'],
            'dbname' if not deprecated_param_name else 'database': testenv['postgresql_db'],
        }
        self.db = psycopg2.connect(**kwargs)

        database_setup_query = """
        DROP TABLE IF EXISTS users;
        CREATE TABLE users(
           id serial PRIMARY KEY,
           name VARCHAR (50),
           password VARCHAR (50),
           email VARCHAR (355),
           created_on TIMESTAMP,
           last_login TIMESTAMP
        );
        INSERT INTO users(name, email) VALUES('kermit', 'kermit@muppets.com');
        DROP FUNCTION IF EXISTS test_proc(VARCHAR(70));
        CREATE FUNCTION test_proc(candidate VARCHAR(70))
        RETURNS text AS $$
        BEGIN
            RETURN(SELECT name FROM users where email = candidate);
        END;
        $$ LANGUAGE plpgsql;
        """
        cursor = self.db.cursor()
        cursor.execute(database_setup_query)
        self.db.commit()
        cursor.close()


        self.cursor = self.db.cursor()
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        tracer.cur_ctx = None

    def tearDown(self):
        if self.cursor and not self.cursor.connection.closed:
          self.cursor.close()
        if self.db and not self.db.closed:
          self.db.close()
        agent.options.allow_exit_as_root = False

    def test_vanilla_query(self):
        self.assertTrue(psycopg2.extras.register_uuid(None, self.db))
        self.assertTrue(psycopg2.extras.register_uuid(None, self.db.cursor()))

        self.cursor.execute("""SELECT * from users""")
        affected_rows = self.cursor.rowcount
        self.assertEqual(1, affected_rows)
        result = self.cursor.fetchone()

        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(0, len(spans))

    def test_basic_query(self):
        with tracer.start_active_span('test'):
            self.cursor.execute("""SELECT * from users""")
            affected_rows = self.cursor.rowcount
            result = self.cursor.fetchone()
            self.db.commit()

        self.assertEqual(1, affected_rows)
        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])
        self.assertEqual(db_span.data["pg"]["user"], testenv['postgresql_user'])
        self.assertEqual(db_span.data["pg"]["stmt"], 'SELECT * from users')
        self.assertEqual(db_span.data["pg"]["host"], testenv['postgresql_host'])
        self.assertEqual(db_span.data["pg"]["port"], testenv['postgresql_port'])

    def test_basic_query_as_root_exit_span(self):
        agent.options.allow_exit_as_root = True
        self.cursor.execute("""SELECT * from users""")
        affected_rows = self.cursor.rowcount
        result = self.cursor.fetchone()
        self.db.commit()

        self.assertEqual(1, affected_rows)
        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))

        db_span = spans[0]

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])
        self.assertEqual(db_span.data["pg"]["user"], testenv['postgresql_user'])
        self.assertEqual(db_span.data["pg"]["stmt"], 'SELECT * from users')
        self.assertEqual(db_span.data["pg"]["host"], testenv['postgresql_host'])
        self.assertEqual(db_span.data["pg"]["port"], testenv['postgresql_port'])

    def test_basic_insert(self):
        with tracer.start_active_span('test'):
            self.cursor.execute("""INSERT INTO users(name, email) VALUES(%s, %s)""", ('beaker', 'beaker@muppets.com'))
            affected_rows = self.cursor.rowcount

        self.assertEqual(1, affected_rows)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])
        self.assertEqual(db_span.data["pg"]["user"], testenv['postgresql_user'])
        self.assertEqual(db_span.data["pg"]["stmt"], 'INSERT INTO users(name, email) VALUES(%s, %s)')
        self.assertEqual(db_span.data["pg"]["host"], testenv['postgresql_host'])
        self.assertEqual(db_span.data["pg"]["port"], testenv['postgresql_port'])

    def test_executemany(self):
        with tracer.start_active_span('test'):
            self.cursor.executemany("INSERT INTO users(name, email) VALUES(%s, %s)",
                                    [('beaker', 'beaker@muppets.com'), ('beaker', 'beaker@muppets.com')])
            affected_rows = self.cursor.rowcount
            self.db.commit()

        self.assertEqual(2, affected_rows)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])
        self.assertEqual(db_span.data["pg"]["user"], testenv['postgresql_user'])
        self.assertEqual(db_span.data["pg"]["stmt"], 'INSERT INTO users(name, email) VALUES(%s, %s)')
        self.assertEqual(db_span.data["pg"]["host"], testenv['postgresql_host'])
        self.assertEqual(db_span.data["pg"]["port"], testenv['postgresql_port'])

    def test_call_proc(self):
        with tracer.start_active_span('test'):
            callproc_result = self.cursor.callproc('test_proc', ('beaker',))

        self.assertIsInstance(callproc_result, tuple)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])
        self.assertEqual(db_span.data["pg"]["user"], testenv['postgresql_user'])
        self.assertEqual(db_span.data["pg"]["stmt"], 'test_proc')
        self.assertEqual(db_span.data["pg"]["host"], testenv['postgresql_host'])
        self.assertEqual(db_span.data["pg"]["port"], testenv['postgresql_port'])

    def test_error_capture(self):
        affected_rows = result = None
        try:
            with tracer.start_active_span('test'):
                self.cursor.execute("""SELECT * from blah""")
                affected_rows = self.cursor.rowcount
                self.cursor.fetchone()
        except Exception:
            pass

        self.assertIsNone(affected_rows)
        self.assertIsNone(result)

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertEqual(1, db_span.ec)
        self.assertEqual(db_span.data["pg"]["error"], 'relation "blah" does not exist\nLINE 1: SELECT * from blah\n                      ^\n')

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])
        self.assertEqual(db_span.data["pg"]["user"], testenv['postgresql_user'])
        self.assertEqual(db_span.data["pg"]["stmt"], 'SELECT * from blah')
        self.assertEqual(db_span.data["pg"]["host"], testenv['postgresql_host'])
        self.assertEqual(db_span.data["pg"]["port"], testenv['postgresql_port'])

    # Added to validate unicode support and register_type.
    def test_unicode(self):
        ext.register_type(ext.UNICODE, self.cursor)
        snowman = "\u2603"

        self.cursor.execute("delete from users where id in (1,2,3)")

        # unicode in statement
        psycopg2.extras.execute_batch(self.cursor,
            "insert into users (id, name) values (%%s, %%s) -- %s" % snowman, [(1, 'x')])
        self.cursor.execute("select id, name from users where id = 1")
        self.assertEqual(self.cursor.fetchone(), (1, 'x'))

        # unicode in data
        psycopg2.extras.execute_batch(self.cursor,
            "insert into users (id, name) values (%s, %s)", [(2, snowman)])
        self.cursor.execute("select id, name from users where id = 2")
        self.assertEqual(self.cursor.fetchone(), (2, snowman))

        # unicode in both
        psycopg2.extras.execute_batch(self.cursor,
            "insert into users (id, name) values (%%s, %%s) -- %s" % snowman, [(3, snowman)])
        self.cursor.execute("select id, name from users where id = 3")
        self.assertEqual(self.cursor.fetchone(), (3, snowman))

    def test_register_type(self):
        import uuid

        oid1 = 2950
        oid2 = 2951

        ext.UUID = ext.new_type((oid1,), "UUID", lambda data, cursor: data and uuid.UUID(data) or None)
        ext.UUIDARRAY = ext.new_array_type((oid2,), "UUID[]", ext.UUID)

        ext.register_type(ext.UUID, self.cursor)
        ext.register_type(ext.UUIDARRAY, self.cursor)

    def test_connect_cursor_ctx_mgr(self):
        with tracer.start_active_span("test"):
            with self.db as connection:
                with connection.cursor() as cursor:
                    cursor.execute("""SELECT * from users""")
                    affected_rows = cursor.rowcount
                    result = cursor.fetchone()

        self.assertEqual(1, affected_rows)
        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv["postgresql_db"])
        self.assertEqual(db_span.data["pg"]["user"], testenv["postgresql_user"])
        self.assertEqual(db_span.data["pg"]["stmt"], "SELECT * from users")
        self.assertEqual(db_span.data["pg"]["host"], testenv["postgresql_host"])
        self.assertEqual(db_span.data["pg"]["port"], testenv["postgresql_port"])

    def test_connect_ctx_mgr(self):
        with tracer.start_active_span("test"):
            with self.db as connection:
                cursor = connection.cursor()
                cursor.execute("""SELECT * from users""")
                affected_rows = cursor.rowcount
                result = cursor.fetchone()

        self.assertEqual(1, affected_rows)
        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv["postgresql_db"])
        self.assertEqual(db_span.data["pg"]["user"], testenv["postgresql_user"])
        self.assertEqual(db_span.data["pg"]["stmt"], "SELECT * from users")
        self.assertEqual(db_span.data["pg"]["host"], testenv["postgresql_host"])
        self.assertEqual(db_span.data["pg"]["port"], testenv["postgresql_port"])

    def test_cursor_ctx_mgr(self):
        with tracer.start_active_span("test"):
            connection = self.db
            with connection.cursor() as cursor:
                cursor.execute("""SELECT * from users""")
                affected_rows = cursor.rowcount
                result = cursor.fetchone()

        self.assertEqual(1, affected_rows)
        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv["postgresql_db"])
        self.assertEqual(db_span.data["pg"]["user"], testenv["postgresql_user"])
        self.assertEqual(db_span.data["pg"]["stmt"], "SELECT * from users")
        self.assertEqual(db_span.data["pg"]["host"], testenv["postgresql_host"])
        self.assertEqual(db_span.data["pg"]["port"], testenv["postgresql_port"])

    def test_deprecated_parameter_database(self):
        """test_deprecated_parameter_database"""

        with tracer.start_active_span('test'):
            self.cursor.execute("""SELECT * from users""")
            affected_rows = self.cursor.rowcount
            result = self.cursor.fetchone()
            self.db.commit()

        self.assertEqual(1, affected_rows)
        self.assertEqual(6, len(result))

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        db_span, test_span = spans

        self.assertEqual("test", test_span.data["sdk"]["name"])
        self.assertEqual(test_span.t, db_span.t)
        self.assertEqual(db_span.p, test_span.s)

        self.assertIsNone(db_span.ec)

        self.assertEqual(db_span.n, "postgres")
        self.assertEqual(db_span.data["pg"]["db"], testenv['postgresql_db'])



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2019


import opentracing
import wrapt
import functools

from ...log import logger
from ...singletons import agent, setup_tornado_tracer, tornado_tracer
from ...util.secrets import strip_secrets_from_query

try:
    import tornado

    # Tornado >=6.0 switched to contextvars for context management.  This requires changes to the opentracing
    # scope managers which we will tackle soon.
    # Limit Tornado version for the time being.
    if not (hasattr(tornado, 'version') and tornado.version[0] < '6'):
        logger.debug('Instana supports Tornado package versions < 6.0.  Skipping.')
        raise ImportError

    setup_tornado_tracer()

    @wrapt.patch_function_wrapper('tornado.httpclient', 'AsyncHTTPClient.fetch')
    def fetch_with_instana(wrapped, instance, argv, kwargs):
        try:
            parent_span = tornado_tracer.active_span

            # If we're not tracing, just return
            if (parent_span is None) or (parent_span.operation_name == "tornado-client"):
                return wrapped(*argv, **kwargs)

            request = argv[0]

            # To modify request headers, we have to preemptively create an HTTPRequest object if a
            # URL string was passed.
            if not isinstance(request, tornado.httpclient.HTTPRequest):
                request = tornado.httpclient.HTTPRequest(url=request, **kwargs)

                new_kwargs = {}
                for param in ('callback', 'raise_error'):
                    # if not in instead and pop
                    if param in kwargs:
                        new_kwargs[param] = kwargs.pop(param)
                kwargs = new_kwargs

            scope = tornado_tracer.start_active_span('tornado-client', child_of=parent_span)
            tornado_tracer.inject(scope.span.context, opentracing.Format.HTTP_HEADERS, request.headers)

            # Query param scrubbing
            parts = request.url.split('?')
            if len(parts) > 1:
                cleaned_qp = strip_secrets_from_query(parts[1], agent.options.secrets_matcher,
                                                      agent.options.secrets_list)
                scope.span.set_tag("http.params", cleaned_qp)

            scope.span.set_tag("http.url", parts[0])
            scope.span.set_tag("http.method", request.method)

            future = wrapped(request, **kwargs)

            if future is not None:
                cb = functools.partial(finish_tracing, scope=scope)
                future.add_done_callback(cb)

            return future
        except Exception:
            logger.debug("tornado fetch", exc_info=True)
            raise


    def finish_tracing(future, scope):
        try:
            response = future.result()
            scope.span.set_tag("http.status_code", response.code)
        except tornado.httpclient.HTTPClientError as e:
            scope.span.set_tag("http.status_code", e.code)
            scope.span.log_exception(e)
            raise
        finally:
            scope.close()


    logger.debug("Instrumenting tornado client")
except ImportError:
    pass



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import time
import unittest
import random
import threading

from instana.autoprofile.profiler import Profiler
from instana.autoprofile.runtime import runtime_info
from instana.autoprofile.samplers.block_sampler import BlockSampler


class BlockSamplerTestCase(unittest.TestCase):
    def test_block_profile(self):
        if runtime_info.OS_WIN:
            return

        profiler = Profiler(None)
        profiler.start(disable_timers=True)
        sampler = BlockSampler(profiler)
        sampler.setup()
        sampler.reset()

        lock = threading.Lock()
        event = threading.Event()

        def lock_lock():
            lock.acquire()
            time.sleep(0.5)
            lock.release()

        def lock_wait():
            lock.acquire()
            lock.release()


        def event_lock():
            time.sleep(0.5)
            event.set()


        def event_wait():
            event.wait()

        def record():
            sampler.start_sampler()
            time.sleep(2)
            sampler.stop_sampler()

        record_t = threading.Thread(target=record)
        record_t.start()

        # simulate lock
        t = threading.Thread(target=lock_lock)
        t.start()

        t = threading.Thread(target=lock_wait)
        t.start()

        # simulate event
        t = threading.Thread(target=event_lock)
        t.start()

        t = threading.Thread(target=event_wait)
        t.start()

        record_t.join()

        profile = sampler.build_profile(2000, 120000).to_dict()
        #print(profile)

        self.assertTrue('lock_wait' in str(profile))
        self.assertTrue('event_wait' in str(profile))


if __name__ == '__main__':
    unittest.main()



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import os
import boto3
import unittest
import urllib3

from moto import mock_aws

import tests.apps.flask_app
from instana.singletons import tracer, agent
from ...helpers import get_first_span_by_filter, testenv

pwd = os.path.dirname(os.path.abspath(__file__))


class TestSqs(unittest.TestCase):
    def setUp(self):
        """ Clear all spans before a test run """
        self.recorder = tracer.recorder
        self.recorder.clear_spans()
        self.mock = mock_aws()
        self.mock.start()
        self.sqs = boto3.client('sqs', region_name='us-east-1')
        self.http_client = urllib3.PoolManager()

    def tearDown(self):
        # Stop Moto after each test
        self.mock.stop()
        agent.options.allow_exit_as_root = False


    def test_vanilla_create_queue(self):
        result = self.sqs.create_queue(
            QueueName='SQS_QUEUE_NAME',
            Attributes={
                'DelaySeconds': '60',
                'MessageRetentionPeriod': '86400'
            })
        self.assertEqual(result['ResponseMetadata']['HTTPStatusCode'], 200)


    def test_send_message(self):
        # Create the Queue:
        response = self.sqs.create_queue(
            QueueName='SQS_QUEUE_NAME',
            Attributes={
                'DelaySeconds': '60',
                'MessageRetentionPeriod': '600'
            }
        )

        self.assertTrue(response['QueueUrl'])
        queue_url = response['QueueUrl']

        with tracer.start_active_span('test'):
            response = self.sqs.send_message(
                QueueUrl=queue_url,
                DelaySeconds=10,
                MessageAttributes={
                    'Website': {
                        'DataType': 'String',
                        'StringValue': 'https://www.instana.com'
                    },
                },
                MessageBody=('Monitor any application, service, or request '
                            'with Instana Application Performance Monitoring')
            )

        self.assertTrue(response['MessageId'])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'SendMessage')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://sqs.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')

        payload = {'QueueUrl': 'https://sqs.us-east-1.amazonaws.com/123456789012/SQS_QUEUE_NAME', 'DelaySeconds': 10,
                'MessageAttributes': {'Website': {'DataType': 'String', 'StringValue': 'https://www.instana.com'}},
                'MessageBody': 'Monitor any application, service, or request with Instana Application Performance Monitoring'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://sqs.us-east-1.amazonaws.com:443/SendMessage')


    def test_send_message_as_root_exit_span(self):
        # Create the Queue:
        response = self.sqs.create_queue(
            QueueName='SQS_QUEUE_NAME',
            Attributes={
                'DelaySeconds': '60',
                'MessageRetentionPeriod': '600'
            }
        )

        self.assertTrue(response['QueueUrl'])
        agent.options.allow_exit_as_root = True
        queue_url = response['QueueUrl']

        response = self.sqs.send_message(
            QueueUrl=queue_url,
            DelaySeconds=10,
            MessageAttributes={
                'Website': {
                    'DataType': 'String',
                    'StringValue': 'https://www.instana.com'
                },
            },
            MessageBody=('Monitor any application, service, or request '
                        'with Instana Application Performance Monitoring')
        )

        self.assertTrue(response['MessageId'])

        spans = self.recorder.queued_spans()
        self.assertEqual(1, len(spans))
        boto_span = spans[0]
        self.assertTrue(boto_span)
        self.assertEqual(boto_span.n, "boto3")
        self.assertIsNone(boto_span.p)
        self.assertIsNone(boto_span.ec)


        self.assertEqual(boto_span.data['boto3']['op'], 'SendMessage')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://sqs.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')

        payload = {'QueueUrl': 'https://sqs.us-east-1.amazonaws.com/123456789012/SQS_QUEUE_NAME', 'DelaySeconds': 10,
                'MessageAttributes': {'Website': {'DataType': 'String', 'StringValue': 'https://www.instana.com'}},
                'MessageBody': 'Monitor any application, service, or request with Instana Application Performance Monitoring'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://sqs.us-east-1.amazonaws.com:443/SendMessage')


    def test_app_boto3_sqs(self):
        with tracer.start_active_span('test'):
            self.http_client.request('GET', testenv["wsgi_server"] + '/boto3/sqs')

        spans = self.recorder.queued_spans()
        self.assertEqual(5, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "urllib3"
        http_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(http_span)

        filter = lambda span: span.n == "wsgi"
        wsgi_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(wsgi_span)

        filter = lambda span: span.n == "boto3" and span.data['boto3']['op'] == 'CreateQueue'
        bcq_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(bcq_span)

        filter = lambda span: span.n == "boto3" and span.data['boto3']['op'] == 'SendMessage'
        bsm_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(bsm_span)

        self.assertEqual(http_span.t, test_span.t)
        self.assertEqual(http_span.p, test_span.s)

        self.assertEqual(wsgi_span.t, test_span.t)
        self.assertEqual(wsgi_span.p, http_span.s)

        self.assertEqual(bcq_span.t, test_span.t)
        self.assertEqual(bcq_span.p, wsgi_span.s)

        self.assertEqual(bsm_span.t, test_span.t)
        self.assertEqual(bsm_span.p, wsgi_span.s)


    def test_request_header_capture_before_call(self):
        # Create the Queue:
        response = self.sqs.create_queue(
            QueueName='SQS_QUEUE_NAME',
            Attributes={
                'DelaySeconds': '60',
                'MessageRetentionPeriod': '600'
            }
        )

        self.assertTrue(response['QueueUrl'])

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This', 'X-Capture-That']

        # Access the event system on the S3 client
        event_system = self.sqs.meta.events

        request_headers = {
                'X-Capture-This': 'this',
                'X-Capture-That': 'that'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_call(params, **kwargs):
            params['headers'].update(request_headers)

        # Register the function to before-call event.
        event_system.register('before-call.sqs.SendMessage', add_custom_header_before_call)

        queue_url = response['QueueUrl']
        with tracer.start_active_span('test'):
            response = self.sqs.send_message(
                QueueUrl=queue_url,
                DelaySeconds=10,
                MessageAttributes={
                    'Website': {
                        'DataType': 'String',
                        'StringValue': 'https://www.instana.com'
                    },
                },
                MessageBody=('Monitor any application, service, or request '
                            'with Instana Application Performance Monitoring')
            )

        self.assertTrue(response['MessageId'])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'SendMessage')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://sqs.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')

        payload = {'QueueUrl': 'https://sqs.us-east-1.amazonaws.com/123456789012/SQS_QUEUE_NAME', 'DelaySeconds': 10,
                'MessageAttributes': {'Website': {'DataType': 'String', 'StringValue': 'https://www.instana.com'}},
                'MessageBody': 'Monitor any application, service, or request with Instana Application Performance Monitoring'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://sqs.us-east-1.amazonaws.com:443/SendMessage')

        self.assertIn("X-Capture-This", boto_span.data["http"]["header"])
        self.assertEqual("this", boto_span.data["http"]["header"]["X-Capture-This"])
        self.assertIn("X-Capture-That", boto_span.data["http"]["header"])
        self.assertEqual("that", boto_span.data["http"]["header"]["X-Capture-That"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_request_header_capture_before_sign(self):
        # Create the Queue:
        response = self.sqs.create_queue(
            QueueName='SQS_QUEUE_NAME',
            Attributes={
                'DelaySeconds': '60',
                'MessageRetentionPeriod': '600'
            }
        )

        self.assertTrue(response['QueueUrl'])

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Custom-1', 'X-Custom-2']

        # Access the event system on the S3 client
        event_system = self.sqs.meta.events

        request_headers = {
                'X-Custom-1': 'Value1',
                'X-Custom-2': 'Value2'
            }

        # Create a function that adds custom headers
        def add_custom_header_before_sign(request, **kwargs):
            for name, value in request_headers.items():
                request.headers.add_header(name, value)

        # Register the function to before-sign event.
        event_system.register_first('before-sign.sqs.SendMessage', add_custom_header_before_sign)

        queue_url = response['QueueUrl']
        with tracer.start_active_span('test'):
            response = self.sqs.send_message(
                QueueUrl=queue_url,
                DelaySeconds=10,
                MessageAttributes={
                    'Website': {
                        'DataType': 'String',
                        'StringValue': 'https://www.instana.com'
                    },
                },
                MessageBody=('Monitor any application, service, or request '
                            'with Instana Application Performance Monitoring')
            )

        self.assertTrue(response['MessageId'])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'SendMessage')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://sqs.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')

        payload = {'QueueUrl': 'https://sqs.us-east-1.amazonaws.com/123456789012/SQS_QUEUE_NAME', 'DelaySeconds': 10,
                'MessageAttributes': {'Website': {'DataType': 'String', 'StringValue': 'https://www.instana.com'}},
                'MessageBody': 'Monitor any application, service, or request with Instana Application Performance Monitoring'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://sqs.us-east-1.amazonaws.com:443/SendMessage')

        self.assertIn("X-Custom-1", boto_span.data["http"]["header"])
        self.assertEqual("Value1", boto_span.data["http"]["header"]["X-Custom-1"])
        self.assertIn("X-Custom-2", boto_span.data["http"]["header"])
        self.assertEqual("Value2", boto_span.data["http"]["header"]["X-Custom-2"])

        agent.options.extra_http_headers = original_extra_http_headers


    def test_response_header_capture(self):
        # Create the Queue:
        response = self.sqs.create_queue(
            QueueName='SQS_QUEUE_NAME',
            Attributes={
                'DelaySeconds': '60',
                'MessageRetentionPeriod': '600'
            }
        )

        self.assertTrue(response['QueueUrl'])

        original_extra_http_headers = agent.options.extra_http_headers
        agent.options.extra_http_headers = ['X-Capture-This-Too', 'X-Capture-That-Too']

        # Access the event system on the S3 client
        event_system = self.sqs.meta.events

        response_headers = {
            "X-Capture-This-Too": "this too",
            "X-Capture-That-Too": "that too",
        }

        # Create a function that sets the custom headers in the after-call event.
        def modify_after_call_args(parsed, **kwargs):
            parsed['ResponseMetadata']['HTTPHeaders'].update(response_headers)

        # Register the function to an event
        event_system.register('after-call.sqs.SendMessage', modify_after_call_args)

        queue_url = response['QueueUrl']
        with tracer.start_active_span('test'):
            response = self.sqs.send_message(
                QueueUrl=queue_url,
                DelaySeconds=10,
                MessageAttributes={
                    'Website': {
                        'DataType': 'String',
                        'StringValue': 'https://www.instana.com'
                    },
                },
                MessageBody=('Monitor any application, service, or request '
                            'with Instana Application Performance Monitoring')
            )

        self.assertTrue(response['MessageId'])

        spans = self.recorder.queued_spans()
        self.assertEqual(2, len(spans))

        filter = lambda span: span.n == "sdk"
        test_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(test_span)

        filter = lambda span: span.n == "boto3"
        boto_span = get_first_span_by_filter(spans, filter)
        self.assertTrue(boto_span)

        self.assertEqual(boto_span.t, test_span.t)
        self.assertEqual(boto_span.p, test_span.s)

        self.assertIsNone(test_span.ec)

        self.assertEqual(boto_span.data['boto3']['op'], 'SendMessage')
        self.assertEqual(boto_span.data['boto3']['ep'], 'https://sqs.us-east-1.amazonaws.com')
        self.assertEqual(boto_span.data['boto3']['reg'], 'us-east-1')

        payload = {'QueueUrl': 'https://sqs.us-east-1.amazonaws.com/123456789012/SQS_QUEUE_NAME', 'DelaySeconds': 10,
                'MessageAttributes': {'Website': {'DataType': 'String', 'StringValue': 'https://www.instana.com'}},
                'MessageBody': 'Monitor any application, service, or request with Instana Application Performance Monitoring'}
        self.assertDictEqual(boto_span.data['boto3']['payload'], payload)

        self.assertEqual(boto_span.data['http']['status'], 200)
        self.assertEqual(boto_span.data['http']['method'], 'POST')
        self.assertEqual(boto_span.data['http']['url'], 'https://sqs.us-east-1.amazonaws.com:443/SendMessage')

        self.assertIn("X-Capture-This-Too", boto_span.data["http"]["header"])
        self.assertEqual("this too", boto_span.data["http"]["header"]["X-Capture-This-Too"])
        self.assertIn("X-Capture-That-Too", boto_span.data["http"]["header"])
        self.assertEqual("that too", boto_span.data["http"]["header"]["X-Capture-That-Too"])

        agent.options.extra_http_headers = original_extra_http_headers



# (c) Copyright IBM Corp. 2021
# (c) Copyright Instana Inc. 2020

import unittest
import sys
import threading
import os

from instana import autoprofile
from instana.autoprofile.profiler import Profiler


class FrameCacheTestCase(unittest.TestCase):

    def test_skip_stack(self):
        profiler = Profiler(None)
        profiler.start(disable_timers=True)
        test_profiler_file = os.path.realpath(autoprofile.__file__)
        self.assertTrue(profiler.frame_cache.is_profiler_frame(test_profiler_file))


if __name__ == '__main__':
    unittest.main()
